@@ -1,7 +1,13 @@
-import pprint, traceback, contextlib, sys
+from __future__ import unicode_literals
+import pprint, traceback, contextlib, os, re
+
+def indent(s, indent=4):
+    ''' Indent the given text.
+        See doctest._indent for the code that inspired this.
+        '''
+    return re.sub('(?m)^(?!$)', indent*' ', s)
 
 def build_where_msg(where, owner, msg=None, use_colors=False):
-    from doctest import _indent
     tmp = []
     try:
         tmp.append('File "%s", line %i' % (where.filepath, where.start_lineno))
@@ -14,7 +20,7 @@ def build_where_msg(where, owner, msg=None, use_colors=False):
         tmp.append((', ' + owner) if tmp else owner)
 
     try:
-        tmp.append('\n' + _indent(highlight_syntax(where, use_colors)))
+        tmp.append('\n' + indent(highlight_syntax(where, use_colors)))
     except Exception as e:
         pass
 
@@ -24,13 +30,17 @@ def build_where_msg(where, owner, msg=None, use_colors=False):
     return ''.join(tmp)
 
 
-def log(msg, x):
-    if x >= 0:
-        print(msg)
+def log(msg, lvl, concerns=None):
+    if isinstance(lvl, int):
+        if lvl >= 0:
+            print(msg)
+
+    else:
+        concerns.event('log', msg=msg, level=lvl)
 
 def colored(s, color, use_colors):
     if use_colors:
-        c = {'green': 32, 'red': 31, 'yellow': 33}[color]
+        c = {'green': 32, 'red': 31, 'yellow': 33, 'cyan': 36}[color]
         return "\033[%sm%s\033[0m" % (c, s)
     else:
         return s
@@ -75,6 +85,10 @@ except ImportError:
         return example.source
 
 def tohuman(s):
+    if isinstance(s, (list, tuple, set)):
+        if isinstance(s, set):
+            s = sorted(list(s))
+        s = ', '.join(s)
     s = s.replace("-", " ").replace("_", " ")
     s = ' '.join(w.capitalize() for w in s.split())
 
@@ -154,7 +168,7 @@ def constant(argumentless_method):
 
 
 @contextlib.contextmanager
-def human_exceptions(where_default, verbosity, quiet, exitcode):
+def human_exceptions(where_default, verbosity, quiet):
     ''' Print to stdout the message of the exception (if any)
         suppressing its traceback.
          - if verbosity is greather than zero, print the traceback.
@@ -163,11 +177,26 @@ def human_exceptions(where_default, verbosity, quiet, exitcode):
         To enhance the print, print the 'where' attribute of the
         exception (if it has one) or the 'where_default'.
 
-        Finally, exit the process with exitcode.
+        If the captured exception is a SystemExit, do not print anything
+        even if verbosity is greather than zero.
+        This allows to use SystemExit as a abort mechanism without printing
+        anything to the console.
+
+        The context manager will yield a dictionary with the
+        key 'exc' set to the exception caught if any or it will
+        remain empty if no exception is caught.
     '''
+    o = {}
     try:
-        yield
-    except Exception as e:
+        yield o
+    except KeyboardInterrupt as e:
+        if not quiet:
+            print('Execution aborted by the user.')
+
+        o['exc'] = e
+    except SystemExit as e:
+        o['exc'] = e
+    except BaseException as e:
         if quiet:
             pass
         else:
@@ -180,16 +209,24 @@ def human_exceptions(where_default, verbosity, quiet, exitcode):
 
             cls = str(e.__class__.__name__)
             print("{where}\n{cls}: {msg}".format(where=where, msg=msg, cls=cls))
-        sys.exit(exitcode)
+
+        o['exc'] = e
 
 @contextlib.contextmanager
 def enhance_exceptions(where, owner, use_colors=False):
     try:
         yield
-    except Exception as e:
+    except BaseException as e:
         if not hasattr(e, 'where'):
             e.where = build_where_msg(where, owner, use_colors=use_colors)
         raise e
 
-
+def abspath(*args):
+    ''' Return the absolute path from the join of <args>.
+        The first item of <args> can be a filename which it will
+        be stripped off to keep just its dirname.
+        '''
+    base = os.path.dirname(args[0])
+    path = os.path.join(base, *args[1:])
+    return os.path.abspath(path)
 
