@@ -1,3 +1,4 @@
+from __future__ import unicode_literals
 from .common import tohuman
 from .options import ExtendOptionParserMixin
 
@@ -28,13 +29,14 @@ class Concern(ExtendOptionParserMixin):
          - start_example
          - end_example
 
-         - user_aborted
          - failure
          - success
-         - crashed
          - timeout
 
+         - crashed
+
          - finally_example
+         - aborted
 
          - start_interact
          - finish_interact
@@ -54,7 +56,7 @@ class Concern(ExtendOptionParserMixin):
         '''
         return parser
 
-    def start(self, examples, runners, filepath):
+    def start(self, examples, runners, filepath, options):
         '''
         Called at the begin of the execution of the given examples
         found in the specific filepath with the given runners.
@@ -149,7 +151,7 @@ class Concern(ExtendOptionParserMixin):
 
         You may expect that eventually one of the following methods will
         be called to mark the end of the execution of the example:
-         - user_aborted
+         - aborted
          - crashed
          - success
          - failure
@@ -203,10 +205,14 @@ class Concern(ExtendOptionParserMixin):
         '''
         pass    # pragma: no cover
 
-    def user_aborted(self, example):
+    def aborted(self, example, by_the_user, options):
         '''
         The given example was cancelled and the run execution
-        aborted by the user, probably with a ctrl-c (^C) or SIGINT.
+        aborted, probably with a ctrl-c (^C) or SIGINT.
+
+        If <by_the_user> is True, the abort was trigger by the user;
+        if it is False, it was done by Byexample, probably after a timeout
+        or a crash.
 
         Keep in mind that the abort could happen before the start or
         after finishing the execution (Concern.start and Concern.finish)
@@ -241,6 +247,13 @@ class Concern(ExtendOptionParserMixin):
         '''
         pass    # pragma: no cover
 
+    def event(self, what, **data):
+        '''
+        Called on arbitrary moments, for arbitrary reasons defined
+        in <what> and optionally in <data>.
+        '''
+        pass    # pragma: no cover
+
 class ConcernComposite(Concern):
     def __init__(self, registry, **unused):
         self.concerns = registry['concerns'].values()
