@@ -1,8 +1,9 @@
+from __future__ import unicode_literals
 import sys, pkgutil, inspect, pprint, os
 
 from .options import Options, OptionParser
 from .runner import ExampleRunner
-from .finder import ExampleHarvest, ExampleFinder
+from .finder import ExampleHarvest, ExampleFinder, ZoneDelimiter
 from .executor import FileExecutor
 from .differ import Differ
 from .parser import ExampleParser
@@ -34,18 +35,29 @@ def are_tty_colors_supported(output):
            'm' not in os.getenv('TERM', '').split('-') and \
            get_colors() >= 8
 
-def is_a(target_class, key_attr):
+def is_a(target_class, key_attr, warn_missing_key_attr):
     '''
     Returns a function that will return True if its argument
     is a subclass of target_class and it has the attribute key_attr
+
+    If warn_missing_key_attr is True, log a warning if the "the argument"
+    is a subclass of target_class but it has not the attribute
+    key_attr.
     '''
     def _is_X(obj):
         if not inspect.isclass(obj):
             return False
 
-        return issubclass(obj, target_class) and \
-               obj is not target_class and \
-               hasattr(obj, key_attr)
+        class_ok = issubclass(obj, target_class) and \
+               obj is not target_class
+
+        attr_ok = hasattr(obj, key_attr)
+
+        if class_ok and not attr_ok:
+            log(" * Warning: class '%s' has not attribute '%s'." % \
+                    (obj.__name__, key_attr),  0)
+
+        return class_ok and attr_ok
 
     return _is_X
 
@@ -55,6 +67,7 @@ def load_modules(dirnames, cfg):
                 'finders': {},
                 'parsers': {},
                 'concerns': {},
+                'zdelimiters': {},
                 }
     for importer, name, is_pkg in pkgutil.iter_modules(dirnames):
         path = importer.path
@@ -69,18 +82,19 @@ def load_modules(dirnames, cfg):
             continue
 
         stability = getattr(module, 'stability', 'undefined')
-        if stability not in ('experimental', 'unstable', 'stable', 'deprecated'):
+        if stability not in ('experimental', 'provisional', 'unstable', 'stable', 'deprecated'):
             stability = 'experimental/%s?' % str(stability)
 
         log("From '%s' loaded '%s' (%s)" % (path, name, stability), verbosity-1)
         for klass, key, what in [(ExampleRunner, 'language', 'runners'),
                                  (ExampleParser, 'language', 'parsers'),
                                  (ExampleFinder, 'target', 'finders'),
+                                 (ZoneDelimiter, 'target', 'zdelimiters'),
                                  (Concern, 'target', 'concerns')]:
 
             # we are interested in any class that is a subclass of 'klass'
             # and it has an attribute 'key'
-            predicate = is_a(klass, key)
+            predicate = is_a(klass, key, verbosity-2)
 
             container = registry[what]
             klasses_found = inspect.getmembers(module, predicate)
@@ -96,7 +110,11 @@ def load_modules(dirnames, cfg):
                 for obj in objs:
                     key_value = getattr(obj, key)
                     if key_value:
-                        container[key_value] = obj
+                        if not isinstance(key_value, (list, tuple, set)):
+                            key_value = [key_value]
+
+                        for k in key_value:
+                            container[k] = obj
                         loaded_objs.append(obj)
 
                 log("\n".join((" - %s" % repr(i)) for i in loaded_objs), verbosity-1)
@@ -120,19 +138,22 @@ def get_allowed_languages(registry, selected):
                                not_found)
     return selected
 
-def get_encoding(encoding, verbosity):
-    if sys.version_info[0] <= 2: # version major
-        # we don't support a different encoding
-        encoding = None
+def verify_encodings(input_encoding, verbosity):
+    if sys.stdout.encoding is None:
+        try:
+            import locale
+            e = (locale.getdefaultlocale()[1] or 'UTF-8')
+        except:
+            e = 'UTF-8'
 
-    log("Encoding: %s." % encoding, verbosity-2)
-    return encoding
+        log(("The encoding of your terminal is unset.\n" +
+                "Try to set the environment variable PYTHONIOENCODING='%s' first\n" +
+                "or run 'byexample' like this:\n" +
+                "  PYTHONIOENCODING='%s' byexample ...\n") % (e, e), verbosity)
+        sys.exit(-1)
 
-def _float_zero_to_none(x):
-    if x == 0:
-        return None
-
-    return float(x)
+    log("Encoding (input): %s." % input_encoding, verbosity-2)
+    log("Encoding (output): %s." % sys.stdout.encoding, verbosity-2)
 
 def geometry(x):
     lines, columns = [int(v.strip()) for v in str(x).split('x')]
@@ -143,21 +164,55 @@ def geometry(x):
 
 def get_default_options_parser(cmdline_args):
     options_parser = OptionParser()
-    options_parser.add_flag("fail-fast", help="if an example fails, fail and stop all the execution.")
-    options_parser.add_flag("norm-ws", help="ignore the amount of whitespaces.")
-    options_parser.add_flag("pass", help="run the example but do not check its output.")
-    options_parser.add_flag("skip", help="do not run the example.")
-    options_parser.add_flag("tags", help="enable the tags <...>.")
-    options_parser.add_flag("enhance-diff", help="improve how the diff are shown.")
-    options_parser.add_flag("interact", help="interact with the runner/interpreter manually if an example fails.")
-    options_parser.add_argument("+rm", action='append', help="remove a character from the got and expected strings.")
-    options_parser.add_argument("+timeout", type=float, help="timeout in seconds to complete the example.")
-    options_parser.add_argument("+diff", choices=['none', 'unified', 'ndiff', 'context'],
-                                        help="select diff algorithm.")
-    options_parser.add_argument("+delaybeforesend", type=_float_zero_to_none,
-                                    help="delay in seconds before sending a line to an runner/interpreter; 0 disable this (default).")
-    options_parser.add_argument("+geometry", type=geometry,
-                                    help="number of lines and columns of the terminal of the form LxC (default to 24x80).")
+    options_parser.add_flag(
+            "fail-fast",
+            default=cmdline_args.fail_fast,
+            help="if an example fails, fail and stop all the execution.")
+    options_parser.add_flag(
+            "norm-ws",
+            default=False,
+            help="ignore the amount of whitespaces.")
+    options_parser.add_flag(
+            "pass",
+            default=False,
+            help="run the example but do not check its output.")
+    options_parser.add_flag(
+            "skip",
+            default=False,
+            help="do not run the example.")
+    options_parser.add_flag(
+            "tags",
+            default=True,
+            help="enable the tags <...>.")
+    options_parser.add_flag(
+            "enhance-diff",
+            default=cmdline_args.enhance_diff,
+            help="improve how the diff are shown.")
+    options_parser.add_argument(
+            "+rm",
+            default=[],
+            action='append',
+            help="remove a character from the got and expected strings.")
+    options_parser.add_argument(
+            "+timeout",
+            default=cmdline_args.timeout,
+            type=float,
+            help="timeout in seconds to complete the example.")
+    options_parser.add_argument(
+            "+diff",
+            default=cmdline_args.diff,
+            choices=['none', 'unified', 'ndiff', 'context'],
+            help="select diff algorithm.")
+    options_parser.add_argument(
+            "+geometry",
+            default=(24, 80),
+            type=geometry,
+            help="number of lines and columns of the terminal of the form LxC (default to 24x80).")
+    options_parser.add_argument(
+            "+term",
+            default='dumb',
+            choices=['as-is', 'dumb', 'ansi'],
+            help="select a terminal emulator to interpret the output (default to 'dumb').")
 
     return options_parser
 
@@ -165,26 +220,18 @@ def get_default_options_parser(cmdline_args):
 def get_options(args, cfg):
     # the options object should have a set of default values based
     # on the flags and values from the command line.
-    options = Options({ 'fail_fast': args.fail_fast,
-                        'norm_ws': False,
-                        'pass': False,
-                        'skip': False,
-                        'tags': True,
-                        'rm': [],
-                        'enhance_diff': args.enhance_diff,
-                        'interact': args.interact,
-                        'timeout': args.timeout,
-                        'diff': args.diff,
-                        'delaybeforesend': None,
+    options = Options({
+                        'interact': False,
                         'shebangs': args.shebangs,
-                        'geometry': (24, 80)
                         })
     log("Options (cmdline): %s" % options, cfg['verbosity']-2)
 
     # create a parser for the rest of the options.
     optparser = get_default_options_parser(args)
     options['optparser'] = optparser
+    options.update(optparser.defaults())
 
+    log("Options (cmdline + byexample's defaults): %s" % options, cfg['verbosity']-2)
     # we parse the argument 'options' to allow the user to change
     # some options.
     #
@@ -207,7 +254,13 @@ def get_options(args, cfg):
     # completed later
     cfg['options']= options
 
-    log("Options (cmdline + --options): %s" % options, cfg['verbosity']-2)
+    log("Options (cmdline + byexample's defaults + --options): %s" % options, cfg['verbosity']-2)
+
+    options['x'] = {}
+    for k, v in vars(args).items():
+        if k.startswith('x_'):
+            options['x'][k[2:]] = v
+
     return options
 
 def show_options(cfg, registry, allowed_languages):
@@ -238,6 +291,12 @@ def extend_option_parser_with_concerns(cfg, registry):
     # starting from the byexample's one
     optparser = cfg['options']['optparser']
     for concern in concerns:
+        # update the options with the concerns's default ones and
+        # only with them; only after this merge concerns's parser with
+        # the previous parser
+        concern_optparser = concern.get_extended_option_parser(parent_parser=None)
+        cfg['options'].update(concern_optparser.defaults())
+
         optparser = concern.get_extended_option_parser(optparser)
 
     cfg['options']['optparser'] = optparser
@@ -247,6 +306,12 @@ def extend_options_with_language_specific(cfg, registry, allowed_languages):
     parsers = [p for p in registry['parsers'].values()
                if p.language in allowed_languages]
 
+    # update the defaults for all the parsers
+    for parser in parsers:
+        parser_optparser = parser.get_extended_option_parser(parent_parser=None)
+        cfg['options'].update(parser_optparser.defaults())
+
+    # update again but with the options that *may* had set in the command line
     for parser in parsers:
         opts = parser.extract_cmdline_options(cfg['opts_from_cmdline'])
         cfg['options'].update(opts)
@@ -255,19 +320,25 @@ def extend_options_with_language_specific(cfg, registry, allowed_languages):
             cfg['verbosity']-2)
 
 def init(args):
-    encoding = get_encoding(args.encoding, args.verbosity)
+    verify_encodings(args.encoding, args.verbosity)
 
     cfg = {
             'use_progress_bar': args.pretty == 'all',
             'use_colors': args.pretty == 'all',
             'quiet':      args.quiet,
             'verbosity':  args.verbosity,
-            'encoding':   encoding,
+            'encoding':   args.encoding,
             'output':     sys.stdout,
-            'interact':   args.interact,
+            'interact':   False,
             'opts_from_cmdline': args.options_str,
             }
 
+    allowed_files = set(args.files) - set(args.skip)
+    testfiles = list(sorted(f for f in args.files if f in allowed_files))
+
+    # Do not spawn more jobs than testfiles
+    args.jobs = cfg['jobs'] = min(args.jobs, len(testfiles))
+
     options = get_options(args, cfg)
 
     # if the output has not color support, disable the color anyways
@@ -277,13 +348,17 @@ def init(args):
 
     allowed_languages = get_allowed_languages(registry, args.languages)
 
-    allowed_files = set(args.files) - set(args.skip)
-    testfiles = [f for f in args.files if f in allowed_files]
-
     if args.show_options:
         show_options(cfg, registry, allowed_languages)
         sys.exit(0)
 
+    if not testfiles:
+        if not cfg['quiet']:
+            log("No files were found (you passed %i files and %i were skipped)" %
+                    (len(set(args.files)), len(set(args.files) - allowed_files)),
+                    cfg['verbosity'])
+        sys.exit(1)
+
     # extend the option parser with all the parsers of the concerns.
     # do this *after* showing the options so we can show each parser's opt
     # separately
