@@ -4,8 +4,12 @@ Example:
   (gdb) print 1 + 2
   $1 = 3
 
+  (gdb) print "hello bla world"     # byexample: +norm-ws
+  $2 = "hello   <...>   world"
+
 """
 
+from __future__ import unicode_literals
 import re, pexpect, sys, time
 from byexample.common import constant
 from byexample.parser import ExampleParser
@@ -89,7 +93,10 @@ class GDBInterpreter(ExampleRunner, PexepctMixin):
                         ]
                     }
 
-    def run(self, example, flags):
+    def run(self, example, options):
+        return PexepctMixin._run(self, example, options)
+
+    def _run_impl(self, example, options):
         if not example.source:
             return ''
 
@@ -98,7 +105,7 @@ class GDBInterpreter(ExampleRunner, PexepctMixin):
         if example.source.endswith('\n'):
             source = example.source[:-1]
 
-        return self._exec_and_wait(source, timeout=int(flags['timeout']))
+        return self._exec_and_wait(source, options)
 
     def interact(self, example, options):
         PexepctMixin.interact(self)
@@ -108,18 +115,22 @@ class GDBInterpreter(ExampleRunner, PexepctMixin):
         shebang = options['shebangs'].get(self.language, shebang)
 
         cmd = ShebangTemplate(shebang).quote_and_substitute(tokens)
-        self._spawn_interpreter(cmd, delaybeforesend=options['delaybeforesend'],
-                                     geometry=options['geometry'])
+        self._spawn_interpreter(cmd, options)
+
+        dfl_timeout = options['x']['dfl_timeout']
 
         # gdb will not print the address of a variable by default
-        self._exec_and_wait('set print address off\n', timeout=1)
+        self._exec_and_wait('set print address off\n', options, timeout=dfl_timeout)
 
         # gdb will stop at the first null when printing an array
-        self._exec_and_wait('set print null-stop on\n', timeout=1)
+        self._exec_and_wait('set print null-stop on\n', options, timeout=dfl_timeout)
 
         # gdb will not ask for "yes or no" confirmation
-        self._exec_and_wait('set confirm off\n', timeout=1)
+        self._exec_and_wait('set confirm off\n', options, timeout=dfl_timeout)
 
     def shutdown(self):
         self._shutdown_interpreter()
 
+    def cancel(self, example, options):
+        return self._abort(example, options)
+
