@@ -1,43 +1,34 @@
-import traceback, time, os, sys
-from byexample.common import colored, highlight_syntax
+from __future__ import unicode_literals
+import traceback, time, os, sys, multiprocessing
+from byexample.common import colored, highlight_syntax, indent
 from byexample.concern import Concern
-from doctest import _indent
-
-stability = 'unstable'
 
 try:
     from tqdm import tqdm
-    if os.environ.get('BYEXAMPLE_PROGRESS_ASCII'):
-        raise ImportError() # fake error
-
+    progress_bar_available = True
 except ImportError:
-    class tqdm(object):
-        def __init__(self, file, *args, **kargs):
-            self.file = file
+    progress_bar_available = False
 
-        def write(self, msg, file, end):
-            file.write(msg)
-            if end:
-                file.write(end)
+stability = 'provisional'
 
-            file.flush()
+class _DummyLock(object):
+    def __enter__(self):
+        return
+    def __exit__(self, *args):
+        pass
 
-        def set_postfix_str(self, msg):
-            pass
+    def acquire(self, *args, **kargs):
+        pass
 
-        def update(self, n):
-            self.file.write('.' * n)
-            self.file.flush()
-
-        def close(self):
-            # do not close the output file
-            self.file.flush()
+    def release(self, *args, **kargs):
+        pass
 
 class SimpleReporter(Concern):
     target = None # progress
 
-    def __init__(self, verbosity, encoding, **unused):
-        if 'use_progress_bar' in unused and unused['use_progress_bar']:
+    def __init__(self, verbosity, encoding, jobs, **unused):
+        if 'use_progress_bar' in unused and unused['use_progress_bar'] \
+                and progress_bar_available:
             self.target = None # disable ourselves
         else:
             self.target = 'progress'
@@ -46,14 +37,24 @@ class SimpleReporter(Concern):
         self.use_colors = unused['use_colors']
         self.verbosity = verbosity
 
+        self.jobs = jobs
+        if self.jobs != 1:
+            self.write_lock = multiprocessing.RLock()
+        else:
+            self.write_lock = _DummyLock()
+
+        self.header_printed = False
+
     def _write(self, msg):
-        self.output.write(msg)
-        self.output.flush()
+        ''' Call me once and just once per concern's method '''
+        with self.write_lock:
+            self.output.write(msg)
+            self.output.flush()
 
     def _update(self, x):
         pass
 
-    def start(self, examples, runners, filepath):
+    def start(self, examples, runners, filepath, options):
         self.num_examples = len(examples)
         self.examplenro = 0
         self.filepath = filepath
@@ -67,7 +68,7 @@ class SimpleReporter(Concern):
                 self._write("File %s, no test found\n" % self.filepath)
             return
 
-        self._write('\n')
+        msg = '\n'
 
         elapsed   = max(time.time() - self.begin, 0)
         if elapsed < 300:
@@ -90,7 +91,7 @@ class SimpleReporter(Concern):
         else:
             status_str = colored("[PASS]", 'green', self.use_colors)
 
-        msg = "File %s, %i/%i test ran in %s\n%s Pass: %i Fail: %i Skip: %i\n" % (
+        msg += "File %s, %i/%i test ran in %s\n%s Pass: %i Fail: %i Skip: %i\n" % (
                     self.filepath,
                     ran_number, tot_number,
                     elapsed_str,
@@ -105,14 +106,21 @@ class SimpleReporter(Concern):
         self.current_merged_flags = options
 
     def start_interact(self, example, options):
-        self._write('\n')
-        self._write("Starting interactive session.\n")
-        self._write("Escape character is '^]'.\n")
+        msg = '\n'
+        msg += "Starting interactive session.\n"
+        msg += "Escape character is '^]'.\n"
+        self._write(msg)
+
+    def _bullet(self, color):
+        return colored("=>", color, self.use_colors)
 
     def timedout(self, example, exception):
-        self._write('\n')
+        self._update(1)
+        msg = '\n'
+        msg += self._error_header(example)
 
-        msg = 'Execution timedout at example %i of %i.\n' % (
+        msg += '%s Execution timedout at example %i of %i.\n' % (
+                                    self._bullet('red'),
                                     self.examplenro, self.num_examples)
         msg += 'This could be because the example just ran too slow (try add more time\n' + \
                'with +timeout=<n>) or the example is "syntactically incorrect" and\n' + \
@@ -121,62 +129,75 @@ class SimpleReporter(Concern):
         if exception.output:
             msg += 'This is the last output obtained:\n%s\n' % str(exception.output)
 
-        self._print_error_header(example)
         self._write(msg)
+        self.fail += 1
 
-    def user_aborted(self, example):
-        self._write('\n')
+    def aborted(self, example, by_the_user, options):
+        msg = '\n'
+        msg += self._error_header(example)
 
-        msg = 'Execution aborted by the user at example %i of %i.\n' % (
+        msg += self._bullet('red') + ' '
+        msg += 'Execution aborted '
+        if by_the_user:
+            msg += 'by the user '
+        msg += 'at example %i of %i.\n' % (
                                     self.examplenro, self.num_examples)
-        self._print_error_header(example)
+
+        msg += 'Some resources may had not been cleaned.\n'
         self._write(msg)
 
     def crashed(self, example, exception):
-        self._write('\n')
+        msg = '\n'
+        msg += self._error_header(example)
 
         tb = ''.join(traceback.format_tb(self._get_traceback(exception)))
         ex = '%s: %s' % (str(exception.__class__.__name__), str(exception))
-        msg = 'Execution of example %i of %i crashed.\n%s\n%s\n' % (
+        msg += self._bullet('red') + ' '
+        msg += 'Execution of example %i of %i crashed.\n%s\n%s\n' % (
                                     self.examplenro, self.num_examples,
                                     tb, ex)
-        self._print_error_header(example)
         self._write(msg)
 
     def start_parse(self, example, options):
+        self.header_printed = False
         self.current_parsing_example = example
+        self.finish_parse_called = False
 
     def finish_parse(self, example, options, exception):
-        self.examplenro += 1
+        if not self.finish_parse_called:
+            self.examplenro += 1
+            self.finish_parse_called = True
 
         if exception == None:
             return
 
-        self._write('\n')
+        msg = '\n'
+        msg += self._error_header(self.current_parsing_example)
 
         ex = '%s: %s' % (str(exception.__class__.__name__), str(exception))
         if self.verbosity >= 1:
             tb = ''.join(traceback.format_tb(self._get_traceback(exception)))
             ex = '\n'.join([tb, ex])
 
-        msg = 'Parse of example %i of %i failed.\n%s\n' % (
+        msg += self._bullet('red') + ' '
+        msg += 'Parse of example %i of %i failed.\n%s\n' % (
                                     self.examplenro, self.num_examples,
                                     ex)
-        self._print_error_header(self.current_parsing_example)
         self._write(msg)
 
     def finish_interact(self, exception):
         if exception == None:
             return
 
-        self._write('\n')
+        msg = '\n'
 
         ex = '%s: %s' % (str(exception.__class__.__name__), str(exception))
         if self.verbosity >= 1:
             tb = ''.join(traceback.format_tb(self._get_traceback(exception)))
             ex = '\n'.join([tb, ex])
 
-        msg = 'Interactive session failed.\n%s\n' % (ex)
+        msg += self._bullet('red') + ' '
+        msg += 'Interactive session failed.\n%s\n' % (ex)
         self._write(msg)
 
     def success(self, example, got, differ):
@@ -185,27 +206,50 @@ class SimpleReporter(Concern):
 
     def failure(self, example, got, differ):
         self._update(1)
-        self._write("\n")
+        msg = "\n"
 
-        self._print_error_header(example)
-        diff = differ.output_difference(example, got, self.current_merged_flags,
+        msg += self._error_header(example)
+        msg += differ.output_difference(example, got, self.current_merged_flags,
                                          self.use_colors)
-        self._write(diff)
-        self._write('\n')
+        msg += '\n'
+        self._write(msg)
 
         self.fail += 1
 
-    def _print_error_header(self, example):
+    def event(self, what, **data):
+        if what == 'log':
+            level = data['level']
+            if level == 'error':
+                hdr = colored("Err:", 'red', self.use_colors)
+            elif level == 'chat':
+                if self.verbosity < 2:
+                    return
+                hdr = colored("Chat:", 'cyan', self.use_colors)
+            elif level == 'warn':
+                hdr = colored("Warn:", 'yellow', self.use_colors)
+            else:
+                return
+
+            msg = "%s %s\n" % (hdr, data['msg'])
+            self._write(msg)
+
+
+    def _error_header(self, example):
+        if self.header_printed:
+            return ''
+
+        self.header_printed = True
         filepath = example.filepath
         lineno = example.start_lineno
 
-        self._write("*" * 70)
+        msg = "*" * 70
 
-        msg = '\nFile "%s", line %i\n' % (filepath, lineno)
-        self._write(msg)
+        msg += '\nFile "%s", line %i\n' % (filepath, lineno)
+        msg += "Failed example:\n"
 
-        self._write("Failed example:\n")
-        self._write(_indent(highlight_syntax(example, self.use_colors)))
+        msg += indent(highlight_syntax(example, self.use_colors))
+
+        return msg
 
     def _get_traceback(self, exception):
         if hasattr(exception, '__traceback__'):
@@ -216,33 +260,68 @@ class SimpleReporter(Concern):
 class ProgressBarReporter(SimpleReporter):
     target = None # progress
 
-    def __init__(self, verbosity, encoding, **unused):
-        SimpleReporter.__init__(self, verbosity, encoding, **unused)
-        if 'use_progress_bar' in unused and not unused['use_progress_bar']:
+    def __init__(self, verbosity, encoding, jobs, **unused):
+        SimpleReporter.__init__(self, verbosity, encoding, jobs, **unused)
+        if ('use_progress_bar' in unused and not unused['use_progress_bar']) \
+                or not progress_bar_available:
             self.target = None # disable ourselves
         else:
             self.target = 'progress'
 
+        self.bar = None
+
+    def _clear_all_bars(self):
+        ''' Based on tqdm.clear method '''
+        # Notes:
+        #  moveto(x)  moves x lines down (insert x new lines \n)
+        #  moveto(-x) moves x lines up (insert x special char to go up)
+        #       all moveto are relative to the current position
+        #  ncols is the columns that the terminal has (assume that all the
+        #       bars have the same size)
+        if not hasattr(self.bar, 'fp'):
+            return
+
+        for pos in range(1, self.jobs+1):
+            self.bar.moveto(pos)
+            self.bar.fp.write('\r' + (' ' * self.bar.ncols)) # clear printing spaces
+            self.bar.fp.write('\r')  # place cursor back at the beginning of line
+            self.bar.moveto(-pos)
+
     def _write(self, msg):
-        self.bar.write(msg, file=self.output, end="")
-        self.output.flush()
+        with self.write_lock:
+            if self.bar is None:
+                SimpleReporter._write(self, msg)
+            else:
+                self._clear_all_bars()
+                self.bar.write(msg, file=self.output, end="")
+                self.output.flush()
 
     def _update(self, x):
         self.bar.update(x)
 
-    def start(self, examples, runners, filepath):
-        SimpleReporter.start(self, examples, runners, filepath)
+    def start(self, examples, runners, filepath, options):
+        if self.jobs == 1:
+            position = None
+        else:
+            # use multiprocessing.Process' id (number) as the position
+            # of its bar
+            position = int(multiprocessing.current_process().name) + 1
+
+        SimpleReporter.start(self, examples, runners, filepath, options)
 
         bar_format = '{desc} |{bar}| [{n_fmt}/{total_fmt}{postfix}]'
         self.bar = tqdm(total=len(examples), file=self.output,
                              desc=filepath, leave=False,
                              bar_format=bar_format,
+                             position=position,
                              disable=None # means disable if the output is not TTY
                              )
+        self.bar.set_lock(self.write_lock)
 
     def finish(self, failed, user_aborted, crashed, broken, timedout):
-        self.bar.close()
         SimpleReporter.finish(self, failed, user_aborted, crashed, broken, timedout)
+        self.bar.close()
+        self.bar = None
 
     def start_example(self, example, options):
         SimpleReporter.start_example(self, example, options)
