@@ -3,22 +3,19 @@ Example:
   >>> def hello():
   ...     print("hello bla world")
 
-  >>> hello()
-  hello<...>world
+  >>> hello()               # byexample: +norm-ws
+  hello   <...>   world
 
-  ```python
-  j = 2
-  for i in range(4):
-      j += i
+  >>> j = 2
+  >>> for i in range(4):
+  ...    j += i
 
-  j + 3
-
-  out:
+  >>> j + 3
   11
-  ```
 
 """
 
+from __future__ import unicode_literals
 import re, pexpect, sys, time
 from byexample.common import log, constant
 from byexample.parser import ExampleParser, ExtendOptionParserMixin
@@ -164,20 +161,20 @@ class PythonParser(ExampleParser):
         Add a few extra options and if self.compatibility_mode is True,
         add all the Python doctest's options.
         '''
-        parser.add_flag("py-doctest", help="enable the compatibility with doctest.")
-        parser.add_flag("py-pretty-print", help="enable the pretty print enhancement.")
-        parser.add_flag("py-remove-empty-lines", help="enable the deletion of empty lines (enabled by default).")
+        parser.add_flag("py-doctest", default=False, help="enable the compatibility with doctest.")
+        parser.add_flag("py-pretty-print", default=True, help="enable the pretty print enhancement.")
+        parser.add_flag("py-remove-empty-lines", default=True, help="enable the deletion of empty lines (enabled by default).")
 
         if getattr(self, 'compatibility_mode', True):
-            parser.add_flag("NORMALIZE_WHITESPACE", help="[doctest] alias for +norm-ws.")
-            parser.add_flag("SKIP", help="[doctest] alias for +skip.")
-            parser.add_flag("ELLIPSIS", help="[doctest] enables the ... wildcard.")
-            parser.add_flag("DONT_ACCEPT_BLANKLINE", help="[doctest] take <BLANKLINE> as literal.")
-            parser.add_flag("DONT_ACCEPT_TRUE_FOR_1", help="[doctest] ignored.")
-            parser.add_flag("IGNORE_EXCEPTION_DETAIL", help="[doctest] ignore the exception details.")
-            parser.add_flag("REPORT_UDIFF", help="[doctest] alias for +diff unified.")
-            parser.add_flag("REPORT_CDIFF", help="[doctest] alias for +diff context.")
-            parser.add_flag("REPORT_NDIFF", help="[doctest] alias for +diff ndiff.")
+            parser.add_flag("NORMALIZE_WHITESPACE", default=False, help="[doctest] alias for +norm-ws.")
+            parser.add_flag("SKIP", default=False, help="[doctest] alias for +skip.")
+            parser.add_flag("ELLIPSIS", default=False, help="[doctest] enables the ... wildcard.")
+            parser.add_flag("DONT_ACCEPT_BLANKLINE", default=False, help="[doctest] take <BLANKLINE> as literal.")
+            parser.add_flag("DONT_ACCEPT_TRUE_FOR_1", default=False, help="[doctest] ignored.")
+            parser.add_flag("IGNORE_EXCEPTION_DETAIL", default=False, help="[doctest] ignore the exception details.")
+            parser.add_flag("REPORT_UDIFF", default=False, help="[doctest] alias for +diff unified.")
+            parser.add_flag("REPORT_CDIFF", default=False, help="[doctest] alias for +diff context.")
+            parser.add_flag("REPORT_NDIFF", default=False, help="[doctest] alias for +diff ndiff.")
 
         return parser
 
@@ -188,32 +185,25 @@ class PythonParser(ExampleParser):
         compatibility_mode = True if original_compatibility_mode == None \
                                   else original_compatibility_mode
 
-        # easy cake, the parsers are already cached
-        if self._optparser_extended_by_comp_mode_cache:
-            return self._optparser_extended_by_comp_mode_cache[compatibility_mode]
-
-        cached = {}
+        tmp = {}
 
         # fake the two compatibility mode (True and False)
         # and build an extended parser for each mode
         self.compatibility_mode = True
-        cached[self.compatibility_mode] = ExtendOptionParserMixin.get_extended_option_parser(
+        tmp[self.compatibility_mode] = ExtendOptionParserMixin.get_extended_option_parser(
                                                 self, parent_parser, **kw)
 
         self.compatibility_mode = False
-        cached[self.compatibility_mode] = ExtendOptionParserMixin.get_extended_option_parser(
+        tmp[self.compatibility_mode] = ExtendOptionParserMixin.get_extended_option_parser(
                                                 self, parent_parser, **kw)
 
-        # save the extended parsers in the cache
-        self._optparser_extended_by_comp_mode_cache = cached
-
         # restore the compatibility mode (even if it was unset)
         if original_compatibility_mode == None:
             del self.compatibility_mode
         else:
             self.compatibility_mode = original_compatibility_mode
 
-        return cached[compatibility_mode]
+        return tmp[compatibility_mode]
 
     def _map_doctest_opts_to_byexample_opts(self):
         '''
@@ -227,7 +217,6 @@ class PythonParser(ExampleParser):
         Return a dictionary with the mapped flags; self.options is unchanged.
         '''
         options = self.options
-        options.mask_default(False)
         mapped = {}
         if options['py_doctest']:
             # map the following doctest's options to byexample's ones
@@ -261,7 +250,6 @@ class PythonParser(ExampleParser):
         if self.options['py_doctest'] and 'tags' not in mapped:
             mapped['tags'] = False
 
-        options.unmask_default()
         return mapped
 
     def _double_parse(self, parse_method, args, kwargs):
@@ -287,7 +275,7 @@ class PythonParser(ExampleParser):
         # the obtained previously (self.options)
         self.options.up(options)
 
-        if self.options.get('py_doctest', False):
+        if self.options['py_doctest']:
             # okay, the user really wanted to be in compatibility mode
             pass
         else:
@@ -330,7 +318,6 @@ class PythonParser(ExampleParser):
 
     def _mutate_expected_based_on_doctest_flags(self, expected_str):
         options = self.options
-        options.mask_default(False)
         if options['py_doctest']:
             if not options['DONT_ACCEPT_BLANKLINE']:
                 expected_str = self._blankline_tag_re.sub('', expected_str)
@@ -340,7 +327,7 @@ class PythonParser(ExampleParser):
                 # we will enable the capture mode, check and warn if the example
                 # contains strings like <label> that may confuse byexample and
                 # or the user
-                if self.capture_tag_regex().search(expected_str):
+                if self.capture_tag_regex()['full'].search(expected_str):
                     log("[Warn] The expected strings has <label> strings that will not be considered literal but as capture tags.",
                             self.verbosity)
 
@@ -385,11 +372,10 @@ class PythonParser(ExampleParser):
                 # enable the capture, this should affect to this example only
                 options['tags'] = True
 
-        options.unmask_default()
         return expected_str
 
     def _remove_empty_line_if_enabled(self, snippet):
-        if self.options.get('py_remove_empty_lines', True):
+        if self.options['py_remove_empty_lines']:
             # remove the empty lines if they are followed by indented lines
             # if they are followed by non-indented lines, the empty lines means
             # "end the block" of code and they should not be removed or we will
@@ -466,6 +452,9 @@ if %s:
                 # unpatch
                 self.builtins_module.repr = self.orig_repr
 
+        def update_width(self, width):
+            self._width = int(width)
+
     __byexample_pretty_print = __ByexamplePrettyPrint(indent=1, width=%i, depth=None)
     del __ByexamplePrettyPrint
 
@@ -487,16 +476,24 @@ del _byexample_pprint
                             "-c", change_prompts,  # run this before anything else
                      ]}
 
-    def run(self, example, flags):
-        return self._exec_and_wait(example.source,
-                                    timeout=int(flags['timeout']))
+    def run(self, example, options):
+        return PexepctMixin._run(self, example, options)
+
+    def _run_impl(self, example, options):
+        return self._exec_and_wait(example.source, options)
+
+    def _change_terminal_geometry(self, rows, cols, options):
+        # update the pretty printer with the new columns value
+        source = '__byexample_pretty_print.update_width(%i)' % cols
+        self._exec_and_wait(source, options, timeout=options['x']['dfl_timeout'])
+        PexepctMixin._change_terminal_geometry(self, rows, cols, options)
 
     def interact(self, example, options):
         PexepctMixin.interact(self)
 
     def initialize(self, options):
-        py_doctest = options.get('py_doctest', False)
-        py_pretty_print = options.get('py_pretty_print', False)
+        py_doctest = options['py_doctest']
+        py_pretty_print = options['py_pretty_print']
         pretty_print = (py_doctest and py_pretty_print) \
                         or not py_doctest
 
@@ -506,8 +503,10 @@ del _byexample_pprint
         cmd = ShebangTemplate(shebang).quote_and_substitute(tokens)
 
         # run!
-        self._spawn_interpreter(cmd, delaybeforesend=options['delaybeforesend'],
-                                     geometry=options['geometry'])
+        self._spawn_interpreter(cmd, options)
 
     def shutdown(self):
         self._shutdown_interpreter()
+
+    def cancel(self, example, options):
+        return self._abort(example, options)
