@@ -6,40 +6,16 @@ Example:
   .. end;
 
   Single line expressions
-  >> hello
-  => "hello<...>world"
+  >> hello               # byexample: +norm-ws
+  => "hello   <...>   world"
 
-  The snippets are detected even if they are inside
-  of a Ruby comment (even if there are out!)
+  >> j = 2
+  >> (0..3).each do |i|
+  ..  j += i
+  .. end
 
-  # >> def say bla
-  # ..   bla
-  # .. end
-  #
-  # >> puts say 'hi'
-  # hi
-  #
-  # >> say 'hello'
-  # => "hello"
-  # >> say 'hello?'
-  # => "hello?"
-  >> puts say 'hello!'
-  hello!
-  >> puts "yes sr!"
-  yes sr!
-
-  Markdown's version (no prompt)
-  ```ruby
-  j = 2;
-  (0..3).each do |i|
-    j += i;
-  end;
-
-  j + 3
-
-  out:
+  >> j + 3
   => 11
-  ```
 
   Pretty print
   >> { 1 => 2, 3=>{4=>"aaaaaaaa", 5=>Array(0..20)}}
@@ -67,26 +43,23 @@ Example:
   "foo bar 4"
 
   Heredocs
-  ```ruby
-  puts <<-FOO
+  >> puts <<-FOO
+  .. one
+  .. two
+  .. FOO
   one
   two
-  FOO
-
-  out:
-  one
-  two
-  ```
 
 """
 
+from __future__ import unicode_literals
 import re, pexpect, sys, time
 from byexample.common import constant
 from byexample.parser import ExampleParser
 from byexample.finder import ExampleFinder
 from byexample.runner import ExampleRunner, PexepctMixin, ShebangTemplate
 
-stability = 'experimental'
+stability = 'provisional'
 
 class RubyPromptFinder(ExampleFinder):
     target = 'ruby-prompt'
@@ -96,14 +69,14 @@ class RubyPromptFinder(ExampleFinder):
         return re.compile(r'''
             # Snippet consists of one PS1 line >> and zero or more PS2 lines
             (?P<snippet>
-                (?:^(?P<indent> [ ]*) (?P<sharp>[#]?)[ ]* >>[ ]   .*)    # PS1 line
-                (?:\n           [ ]*  (?P=sharp)[ ]*     \.\.    .*)*)  # zero or more PS2 lines
+                (?:^(?P<indent> [ ]*) >>[ ]   .*)    # PS1 line
+                (?:\n           [ ]*  \.\.    .*)*)  # zero or more PS2 lines
             \n?
             # Want consists of any non-blank lines that do not start with PS1
             # The '=>' indicator is included (implicitly) and may not exist
-            (?P<expected> (?:(?![ ]*$)                    # Not a blank line
-                             (?![ ]* (?: [#]?[ ]*)  >>)   # Not a line starting with PS1
-                             [ ]* (?P=sharp) .+$\n?       # But any other line with the same prefix
+            (?P<expected> (?:(?![ ]*$)            # Not a blank line
+                             (?![ ]*   >>)        # Not a line starting with PS1
+                             .+$\n?               # But any other line
                       )*)
             ''', re.MULTILINE | re.VERBOSE)
 
@@ -134,7 +107,7 @@ class RubyParser(ExampleParser):
                                                     re.MULTILINE)
 
     def extend_option_parser(self, parser):
-        parser.add_flag("ruby-pretty-print", help="enable the pretty print enhancement.")
+        parser.add_flag("ruby-pretty-print", default=True, help="enable the pretty print enhancement.")
         parser.add_argument("+ruby-expr-print", choices=['auto', 'true', 'false'],
                             default='auto',
                             help='print the expression\'s value (true); ' +\
@@ -152,8 +125,10 @@ class RubyInterpreter(ExampleRunner, PexepctMixin):
 
         self.encoding = encoding
 
-    def run(self, example, flags):
+    def run(self, example, options):
+        return PexepctMixin._run(self, example, options)
 
+    def _run_impl(self, example, options):
         # turn on/off the echo mode base on the setting from the
         # start; per example setting is not supported
         src = example.source
@@ -161,14 +136,14 @@ class RubyInterpreter(ExampleRunner, PexepctMixin):
         if self.expr_print_mode == 'auto':
             if self._detect_expression_print_expected(example):
                 print_expr = True
-                src = 'IRB.CurrentContext.echo = true;\n' + src
+                src = 'IRB.CurrentContext.echo = true; ' + src
             else:
                 print_expr = False
-                src = 'IRB.CurrentContext.echo = false;\n' + src
+                src = 'IRB.CurrentContext.echo = false; ' + src
 
         # there is no need to revert the echo=True if it was changed
         # because the execution of the next example will set it correctly
-        return self._exec_and_wait(src, timeout=int(flags['timeout']))
+        return self._exec_and_wait(src, options)
 
     _EXPR_RESULT_RE = re.compile(r'^=>( |$)', re.MULTILINE | re.DOTALL)
 
@@ -184,11 +159,11 @@ class RubyInterpreter(ExampleRunner, PexepctMixin):
         return  "%e %p %a", {
                     'e': '/usr/bin/env',
                     'p': 'irb',
-                    'a': []
+                    'a': ['--noreadline']
                     }
 
     def initialize(self, options):
-        ruby_pretty_print = options.get('ruby_pretty_print', True)
+        ruby_pretty_print = options['ruby_pretty_print']
 
         # always/yes; never/no; autoetect normalization
         self.expr_print_mode = options['ruby_expr_print']
@@ -198,22 +173,63 @@ class RubyInterpreter(ExampleRunner, PexepctMixin):
 
         cmd = ShebangTemplate(shebang).quote_and_substitute(tokens)
 
+        dfl_timeout = options['x']['dfl_timeout']
+
         # run!
-        self._spawn_interpreter(cmd, delaybeforesend=options['delaybeforesend'],
-                                     geometry=options['geometry'])
+        self._spawn_interpreter(cmd, options, wait_first_prompt=False)
+
+        # In RVM contexts the IRB's prompt mode is changed even
+        # if we force the mode from the command line (whe we spawn IRB)
+        # Make sure that the first thing executed restores the default prompt.
+        self._exec_and_wait(
+                'IRB.CurrentContext.prompt_mode = :DEFAULT\n',
+                options,
+                timeout=dfl_timeout)
+        self._drop_output()
 
         # set the pretty print inspector
         if ruby_pretty_print:
-            self._exec_and_wait('IRB.CurrentContext.inspect_mode = :pp\n',
-                                    timeout=2)
+            self._exec_and_wait(_pp_code, options, timeout=dfl_timeout)
 
         # disable the echo if we don't want it (false) or we may want it
         # but it will depend on the example (auto)
         if self.expr_print_mode in ('auto', 'false'):
-            self._exec_and_wait('IRB.CurrentContext.echo = false\n', timeout=2)
+            self._exec_and_wait('IRB.CurrentContext.echo = false\n',
+                                    options, timeout=dfl_timeout)
         else:
-            self._exec_and_wait('IRB.CurrentContext.echo = true\n', timeout=2)
+            self._exec_and_wait('IRB.CurrentContext.echo = true\n',
+                                    options, timeout=dfl_timeout)
 
     def shutdown(self):
         self._shutdown_interpreter()
 
+    def cancel(self, example, options):
+        return self._abort(example, options)
+
+
+'''
+The following code is followed by several tabs (do not delete them).
+In 7.4.3 and before, this tabs were interpreted by IRB and triggered
+the 'autocomplete' suggestions, printing a lot of stuff to the terminal.
+Now this should not happen anymore so the following code should print
+nothing.
+>> a = "hello"				
+
+'''
+
+_pp_code = '''
+require "pp"
+
+class PP
+  module PPMethods
+    alias_method :_original_pp_hash, :pp_hash
+
+    def pp_hash(obj)
+      obj = Hash[obj.sort]
+      _original_pp_hash(obj)
+    end
+  end
+end
+
+IRB.CurrentContext.inspect_mode = :pp
+'''
