@@ -4,22 +4,19 @@ Example:
   >     echo "hello bla world"
   > }
 
-  $ hello
-  hello<...>world
+  $ hello               # byexample: +norm-ws
+  hello   <...>   world
 
-  ```shell
-  for i in 0 1 2 3; do
-      echo $i
-  done
-
-  out:
+  $ for i in 0 1 2 3; do
+  >    echo $i
+  > done
   0
   1
   2
   3
-  ```
 """
 
+from __future__ import unicode_literals
 import re, pexpect, sys, time
 from byexample.common import constant
 from byexample.parser import ExampleParser
@@ -27,7 +24,7 @@ from byexample.finder import ExampleFinder
 from byexample.runner import ExampleRunner, PexepctMixin, ShebangTemplate
 from byexample.executor import TimeoutException
 
-stability = 'unstable'
+stability = 'provisional'
 
 class ShellPromptFinder(ExampleFinder):
     target = 'shell-prompt'
@@ -68,7 +65,14 @@ class ShellParser(ExampleParser):
                                                     re.MULTILINE)
 
     def extend_option_parser(self, parser):
-        parser.add_flag("stop-on-silence", help="stop the process after some period of inactivity or silence.")
+        parser.add_flag("stop-on-timeout", default=False, help="stop the process if it timeout.")
+        parser.add_argument(
+                "+stop-on-silence",
+                nargs='?',
+                default=False,
+                const=0.2,
+                type=float,
+                help="stop the process if it timeout.")
 
 class ShellInterpreter(ExampleRunner, PexepctMixin):
     language = 'shell'
@@ -87,12 +91,16 @@ class ShellInterpreter(ExampleRunner, PexepctMixin):
                     'a': [],
                     }
 
-    def run(self, example, flags):
+    def run(self, example, options):
+        return PexepctMixin._run(self, example, options)
+
+    def _run_impl(self, example, options):
+        stop_on_timeout = options['stop_on_timeout'] is not False
+        stop_on_silence = options['stop_on_silence'] is not False
         try:
-            return self._exec_and_wait(example.source,
-                                    timeout=int(flags['timeout']))
+            return self._exec_and_wait(example.source, options)
         except TimeoutException as ex:
-            if 'stop_on_silence' in flags and flags['stop_on_silence']:
+            if stop_on_timeout or stop_on_silence:
                 # get the current output
                 out = ex.output
 
@@ -102,12 +110,40 @@ class ShellInterpreter(ExampleRunner, PexepctMixin):
                 self.interpreter.sendcontrol('z')
 
                 # wait for the prompt, ignore any extra output
-                self._expect_prompt(timeout=int(flags['timeout']),
-                                        prompt_re=self.PS1_re)
+                self._expect_prompt(
+                        options,
+                        timeout=options['x']['dfl_timeout'],
+                        prompt_re=self.PS1_re)
+
                 self._drop_output()
                 return out
             raise
 
+    def _expect_prompt(self, options, timeout, prompt_re=None):
+        if options['stop_on_silence'] is not False:
+            silence_timeout = options['stop_on_silence']
+            prev = 0
+            while 1:
+                try:
+                    begin = time.time()
+                    return PexepctMixin._expect_prompt(self, options, silence_timeout, prompt_re)
+                except TimeoutException as ex:
+                    timeout -= max(time.time() - begin, 0)
+                    silence_timeout = min(silence_timeout, timeout)
+
+                    # a real timeout
+                    if timeout <= 0 or silence_timeout <= 0:
+                        raise
+
+                    # inactivity or silence detected
+                    if prev >= len(ex.output):
+                        raise
+
+                    prev = len(ex.output)
+
+        else:
+            return PexepctMixin._expect_prompt(self, options, timeout, prompt_re)
+
     def interact(self, example, options):
         PexepctMixin.interact(self)
 
@@ -116,18 +152,19 @@ class ShellInterpreter(ExampleRunner, PexepctMixin):
         shebang = options['shebangs'].get(self.language, shebang)
 
         cmd = ShebangTemplate(shebang).quote_and_substitute(tokens)
-        self._spawn_interpreter(cmd, wait_first_prompt=False,
-                                delaybeforesend=options['delaybeforesend'],
-                                geometry=options['geometry'])
+        self._spawn_interpreter(cmd, options, wait_first_prompt=False)
 
         self._exec_and_wait(
 '''export PS1="/byexample/sh/ps1> "
 export PS2="/byexample/sh/ps2> "
 export PS3="/byexample/sh/ps3> "
 export PS4="/byexample/sh/ps4> "
-''', timeout=10)
+''', options, timeout=options['x']['dfl_timeout'])
         self._drop_output() # discard banner and things like that
 
     def shutdown(self):
         self._shutdown_interpreter()
 
+    def cancel(self, example, options):
+        return self._abort(example, options)
+
